@@ -51,6 +51,7 @@
 #include "mount.h"
 #include "type.h"
 #include "xtypes.h"
+#include "log.h"
 
 #if defined(HAVE_GETXATTR)
 # define ATTR_MAN_PAGE N_("See the attr(5) man page for full details.")
@@ -124,6 +125,7 @@ static Option o_action_newer;
 
 static Option o_action_mount_command;
 static Option o_action_umount_command;
+static Option o_action_eject_command;
 
 /* Whenever the text in these boxes is changed we store a copy of the new
  * string to be used as the default next time.
@@ -969,7 +971,7 @@ static void do_delete(const char *src_path, const char *unused)
 
 static void do_eject(const char *path)
 {
-	const char *argv[3] = {NULL, NULL, NULL};
+	const char *argv[]={"sh", "-c", NULL, NULL};
 	char *err;
 	
 	check_flags();
@@ -1000,10 +1002,10 @@ static void do_eject(const char *path)
 		g_return_if_fail(c == 'X');
 	}
 
-	argv[0] = "eject";
-	argv[1] = path;
-	argv[2] = NULL;
-	err = fork_exec_wait(argv);
+	argv[2] = build_command_with_path(o_action_eject_command.value,
+					  path);
+	err = fork_exec_wait((const char**)argv);
+	g_free((gchar *) argv[2]);
 	if (err)
 	{
 		printf_send(_("!%s\neject failed\n"), err);
@@ -1204,8 +1206,12 @@ static void do_settype(const char *path, const char *unused)
 		int res;
 		printf_send("<%s", path);
 		printf_send(">");
-		res=printf_reply(from_parent, FALSE,
-				 _("?Change type of '%s'?"), path);
+		if (S_ISDIR(info.st_mode))
+			res=printf_reply(from_parent, FALSE,
+					 _("?Change contents of '%s'?"), path);
+		else
+			res=printf_reply(from_parent, FALSE,
+					 _("?Change type of '%s'?"), path);
 		printf_send("<");
 		if (!res)
 			return;
@@ -1265,6 +1271,16 @@ static void do_settype(const char *path, const char *unused)
 			for_dir_contents(do_settype, safe_path, unused);
 			g_free(safe_path);
 		}
+		else if(!o_brief)
+		{
+			printf_send(_("'Not changing type of directory '%s'\n"),
+				    path);
+		}
+	}
+	else if(!o_brief)
+	{
+		printf_send(_("'Non-regular file '%s' not changed\n"),
+			    path);
 	}
 }
 
@@ -1607,14 +1623,14 @@ static void do_link_absolute(const char *path, const char *dest)
 /* Mount/umount this item (depending on 'mount') */
 static void do_mount(const guchar *path, gboolean mount)
 {
-	const char *argv[3] = {NULL, NULL, NULL};
+	const char *argv[] = {"sh", "-c", NULL, NULL};
 	char *err;
 
 	check_flags();
 
-	argv[0] = mount ? o_action_mount_command.value
-			: o_action_umount_command.value;
-	argv[1] = path;
+	argv[2] = build_command_with_path(mount ? o_action_mount_command.value
+					  : o_action_umount_command.value,
+					  path);
 
 	if (quiet)
 		printf_send(mount ? _("'Mounting %s\n")
@@ -1639,6 +1655,7 @@ static void do_mount(const guchar *path, gboolean mount)
 	}
 
 	err = fork_exec_wait(argv);
+	g_free((gchar *) argv[2]);
 	if (err)
 	{
 		printf_send(mount ?
@@ -2033,6 +2050,8 @@ void action_mount(GList	*paths, gboolean open_dir, gboolean mount, int quiet)
 	if (!gui_side)
 		return;
 
+	log_info_paths("Mount", paths, NULL);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 #else
@@ -2069,6 +2088,8 @@ void action_delete(GList *paths)
 		_("Brief"), _("Only log directories being deleted"),
 		'B', o_action_brief.int_value);
 
+	log_info_paths("Delete", paths, NULL);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
@@ -2141,6 +2162,8 @@ void action_chmod(GList *paths, gboolean force_recurse, const char *action)
 			gui_side->yes);
 #endif
 
+	log_info_paths("Change permissions", paths, NULL);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 
@@ -2193,7 +2216,7 @@ void action_settype(GList *paths, gboolean force_recurse, const char *oldtype)
 	gui_side->default_string = &last_settype_string;
 
 	/* Note: get the list again each time -- it can change */
-	presets = mime_type_name_list();
+	presets = mime_type_name_list(TRUE);
 	abox_add_combo(ABOX(abox), _("Type:"), presets, new_entry_string,
 				new_help_button(show_settype_help, NULL));
 	g_list_free(presets);
@@ -2201,6 +2224,8 @@ void action_settype(GList *paths, gboolean force_recurse, const char *oldtype)
 	g_signal_connect(ABOX(abox)->entry, "changed",
 			G_CALLBACK(entry_changed), gui_side);
 
+	log_info_paths("Set file type", paths, NULL);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 
@@ -2208,6 +2233,22 @@ out:
 	null_g_free(&new_entry_string);
 }
 
+static void log_info_paths_leaf(const gchar *message, GList *paths,
+				const gchar *dest, const char *leaf)
+{
+	if (leaf == NULL)
+	{
+		log_info_paths(message, paths, dest);
+	}
+	else
+	{
+		char *new_dest;
+		new_dest = g_build_filename(dest, leaf, NULL);
+		log_info_paths(message, paths, new_dest);
+		g_free(new_dest);
+	}
+}
+
 /* If leaf is NULL then the copy has the same name as the original.
  * quiet can be -1 for default.
  */
@@ -2242,6 +2283,8 @@ void action_copy(GList *paths, const char *dest, const char *leaf, int quiet)
 		_("Brief"), _("Only log directories as they are copied"),
 		'B', o_action_brief.int_value);
 
+	log_info_paths_leaf("Copy", paths, dest, leaf);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
@@ -2279,6 +2322,9 @@ void action_move(GList *paths, const char *dest, const char *leaf, int quiet)
 	abox_add_flag(ABOX(abox),
 		_("Brief"), _("Don't log each file as it is moved"),
 		'B', o_action_brief.int_value);
+
+	log_info_paths_leaf("Move", paths, dest, leaf);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
@@ -2308,6 +2354,8 @@ void action_link(GList *paths, const char *dest, const char *leaf,
 	if (!gui_side)
 		return;
 
+	log_info_paths_leaf("Link", paths, dest, leaf);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
@@ -2329,6 +2377,8 @@ void action_eject(GList *paths)
 	if (!gui_side)
 		return;
 
+	log_info_paths("Eject", paths, NULL);
+
 	number_of_windows++;
 	gtk_widget_show(abox);
 }
@@ -2349,6 +2399,8 @@ void action_init(void)
 			  "action_mount_command", "mount");
 	option_add_string(&o_action_umount_command,
 			  "action_umount_command", "umount");
+	option_add_string(&o_action_eject_command,
+			  "action_eject_command", "eject");
 }
 
 #define MAX_ASK 4
