@@ -298,11 +298,13 @@ GtkWidget *options_show(void)
 	if (!option_tooltips)
 		option_tooltips = gtk_tooltips_new();
 
+	/* For debugging
 	if (g_hash_table_size(loading) != 0)
 	{
 		g_print(PROJECT ": Some options loaded but not used:\n");
 		g_hash_table_foreach(loading, (GHFunc) puts, NULL);
 	}
+	*/
 
 	if (window)
 	{
@@ -835,8 +837,6 @@ static void null_widget(gpointer key, gpointer value, gpointer data)
 {
 	Option	*option = (Option *) value;
 
-	g_return_if_fail(option->widget != NULL);
-
 	option->widget = NULL;
 }
 
@@ -1181,10 +1181,8 @@ static void update_cb(gpointer key, gpointer value, gpointer data)
 	Option *option = (Option *) value;
 
 	g_return_if_fail(option != NULL);
-	if (option->widget == NULL) {
-		g_warning("No widget for option %s", (char *) key);
+	if (option->widget == NULL)
 		return;
-	}
 
 	updating_widgets++;
 	
