@@ -38,6 +38,7 @@
 #include "diritem.h"
 #include "action.h"
 #include "icon.h"
+#include "choices.h"
 
 /* Static prototypes */
 static void write_data(gpointer data, gint fd, GdkInputCondition cond);
@@ -323,6 +324,62 @@ gboolean run_by_path(const guchar *full_path)
 	return retval;
 }
 
+/* Convert uri to path and call run_by_path() */
+gboolean run_by_uri(const gchar *uri, gchar **errmsg)
+{
+	gboolean retval;
+	gchar *tmp, *tmp2;
+	gchar *scheme;
+	gchar *cmd;
+
+	scheme=get_uri_scheme((EscapedPath *) uri);
+	if(!scheme)
+	{
+		*errmsg=g_strdup_printf(_("'%s' is not a valid URI"),
+						uri);
+		return FALSE;
+	}
+
+	if(strcmp(scheme, "file")==0) {
+		tmp=get_local_path((EscapedPath *) uri);
+		if(tmp) {
+			tmp2=pathdup(tmp);
+			retval=run_by_path(tmp2);
+			if(!retval)
+				*errmsg=g_strdup_printf(_("%s not accessable"),
+							tmp);
+		
+			g_free(tmp2);
+			g_free(tmp);
+
+		} else {
+			retval=FALSE;
+			*errmsg=g_strdup_printf(_("Non-local URL %s"), uri);
+		}
+
+	} else if((cmd=choices_find_xdg_path_load(scheme, "URI", SITE))) {
+		DirItem *item;
+
+		item=diritem_new(scheme);
+		diritem_restat(cmd, item, NULL);
+
+		run_with_args(cmd, item, uri);
+		retval=TRUE; /* we hope... */
+
+		diritem_free(item);
+		g_free(cmd);
+		
+	} else {
+		retval=FALSE;
+		*errmsg=g_strdup_printf(_("%s: no handler for %s"),
+					uri, scheme);
+	}
+	
+	g_free(scheme);
+
+	return retval;
+}
+
 /* Open dir/Help, or show a message if missing */
 void show_help_files(const char *dir)
 {
@@ -563,6 +620,7 @@ static gboolean run_desktop(const char *full_path,
 {
 	GError *error = NULL;
 	char *exec = NULL;
+	char *terminal = NULL;
 	gint argc = 0;
 	gchar **argv = NULL;
 	GPtrArray *expanded = NULL;
@@ -570,8 +628,11 @@ static gboolean run_desktop(const char *full_path,
 	int i;
 	gboolean success = FALSE;
 
-	exec = get_value_from_desktop_file(full_path, "Desktop Entry", "Exec",
-					&error);
+	get_values_from_desktop_file(full_path,
+					&error,
+					"Desktop Entry", "Exec", &exec,
+					"Desktop Entry", "Terminal", &terminal,
+					NULL);
 	if (error)
 	{
 		delayed_error("Failed to parse .desktop file '%s':\n%s",
@@ -594,6 +655,12 @@ static gboolean run_desktop(const char *full_path,
 	}
 
 	expanded = g_ptr_array_new();
+
+	if (terminal && g_strcasecmp(terminal, "true") == 0) {
+		g_ptr_array_add(expanded, g_strdup("xterm"));
+		g_ptr_array_add(expanded, g_strdup("-e"));
+	}
+
 	for (i = 0; i < argc; i++)
 	{
 		const char *src = argv[i];
@@ -638,6 +705,8 @@ err:
 		g_error_free(error);
 	if (exec != NULL)
 		g_free(exec);
+	if (terminal != NULL)
+		g_free(terminal);
 	if (argv != NULL)
 		g_strfreev(argv);
 	if (expanded != NULL)
