@@ -521,6 +521,28 @@ char *get_local_path(const EscapedPath *escaped_uri)
 	}
 }
 
+/* Return the scheme part of a URI, or NULL if not a valid URI (see
+ * RFC 2396).  g_free() the returned value.
+ */
+#define SCHEME_CHARS "+-."  /* In addition to alpha-numerics */
+gchar *get_uri_scheme(const EscapedPath *uri)
+{
+	const gchar *start=(const gchar *) uri;
+	const gchar *p=start;
+	
+	if(!g_ascii_isalpha(p[0]))
+		return NULL;
+
+	while(g_ascii_isalnum(p[0]) ||
+	      strchr(SCHEME_CHARS, p[0]))
+		p++;
+
+	if(p[0]!=':')
+		return NULL;
+
+	return g_strndup(start, p-start);
+}
+
 /* Set the close-on-exec flag for this FD.
  * TRUE means that an exec()'d process will not get the FD.
  */
@@ -1597,7 +1619,7 @@ err:
 
 /* Load .desktop file 'path' and set the value of the named keys in the
  * NULL terminated list.
- * Sets error if the desktop file cannot be parsed and returns NULL.
+ * Sets error if the desktop file cannot be parsed and returns false.
  * Sets NULL (but does not set error) if the key is not present.
  * String set must be g_free()d.
  */
@@ -1658,3 +1680,40 @@ err:
 
 	return (*error==NULL);
 }
+
+/**
+ * Build a command to execute with a supplied path.  If the command pattern
+ * given contains @c $1 it is substituted with the path, otherwise the path
+ * is appended to the pattern after a space character
+ * @param[in] cmd command pattern, with optional @c $1
+ * @param[in] path path to substitute or append
+ * @return generated command line, pass to g_free() when done.
+ */
+gchar *build_command_with_path(const char *cmd, const char *path)
+{
+	const char *subs;
+	gchar *result;
+
+	for(subs=cmd; *subs; subs++)
+	{
+		if(subs[0]=='\\' && subs[1]=='$')
+		{
+			subs++;
+			continue;
+		}
+		if(strncmp(subs, "$1", 2)==0)
+			break;
+	}
+	if(*subs) {
+		int size=strlen(cmd)+strlen(path)+1;
+		result = g_new(char, size);
+		strncpy(result, cmd, subs-cmd);
+		result[subs-cmd] = 0;
+		strcat(result, path);
+		strcat(result, subs+2);
+	} else {
+		result = g_strconcat(cmd, " ", path, NULL);
+	}
+
+	return result;
+}
