@@ -58,6 +58,7 @@ int stat_with_timeout(const char *path, struct stat *info);
 EscapedPath *escape_uri_path(const char *path);
 EscapedPath *encode_path_as_uri(const guchar *path);
 gchar *unescape_uri(const EscapedPath *uri);
+gchar *get_uri_scheme(const EscapedPath *uri);
 gboolean available_in_path(const char *file);
 char *get_value_from_desktop_file(const char *path,
 				  const char *section,
@@ -68,5 +69,6 @@ gboolean get_values_from_desktop_file(const char *path,
 				      const char *section,
 				      const char *key,
 				      gchar **value, ...);
+gchar *build_command_with_path(const char *cmd, const char *path);
 
 #endif /* _SUPPORT_H */
