@@ -117,7 +117,9 @@ MIME_type *inode_door;
 static Option o_display_colour_types;
 static Option o_icon_theme;
 
-GtkIconTheme *icon_theme = NULL;
+static GtkIconTheme *icon_theme = NULL;
+static GtkIconTheme *rox_theme = NULL;
+static GtkIconTheme *gnome_theme = NULL;
 
 void type_init(void)
 {
@@ -185,7 +187,12 @@ static MIME_type *get_mime_type(const gchar *type_name, gboolean can_create)
 		return mtype;
 
 	slash = strchr(type_name, '/');
-	g_return_val_if_fail(slash != NULL, NULL);     /* XXX: Report nicely */
+	if (slash == NULL)
+	{
+		g_warning("MIME type '%s' does not contain a '/' character!",
+			  type_name);
+		return NULL;
+	}
 
 	mtype = g_new(MIME_type, 1);
 	mtype->media_type = g_strndup(type_name, slash - type_name);
@@ -231,24 +238,34 @@ const char *basetype_name(DirItem *item)
 	return _("Unknown");
 }
 
+struct mime_list {
+	GList *list;
+	gboolean only_regular;
+};
+
 static void append_names(gpointer key, gpointer value, gpointer udata)
 {
-	GList **list = (GList **) udata;
+	struct mime_list *mlist = (struct mime_list*) udata;
 
-	*list = g_list_prepend(*list, key);
+	if(!mlist->only_regular || strncmp((char *)key, "inode/", 6)!=0)
+		mlist->list = g_list_prepend(mlist->list, key);
 }
 
 /* Return list of all mime type names. Caller must free the list
  * but NOT the strings it contains (which are never freed).
+ If only_regular is true then inode types are excluded.
  */
-GList *mime_type_name_list(void)
+GList *mime_type_name_list(gboolean only_regular)
 {
-	GList *list = NULL;
+	struct mime_list list;
 
+	list.list=NULL;
+	list.only_regular=only_regular;
+		
 	g_hash_table_foreach(type_hash, append_names, &list);
-	list = g_list_sort(list, (GCompareFunc) strcmp);
+	list.list = g_list_sort(list.list, (GCompareFunc) strcmp);
 
-	return list;
+	return list.list;
 }
 
 /*			MIME-type guessing 			*/
@@ -298,7 +315,7 @@ MIME_type *type_from_path(const char *path)
 		return mime_type;
 
 	/* Try name and contents next */
-	type_name = xdg_mime_get_mime_type_for_file(path);
+	type_name = xdg_mime_get_mime_type_for_file(path, NULL);
 	if (type_name)
 		return get_mime_type(type_name, TRUE);
 
@@ -362,6 +379,64 @@ MIME_type *mime_type_lookup(const char *type)
 	return get_mime_type(type, TRUE);
 }
 
+static void init_aux_theme(GtkIconTheme **ptheme, const char *name)
+{
+	if (*ptheme)
+		return;
+	*ptheme = gtk_icon_theme_new();
+	gtk_icon_theme_set_custom_theme(*ptheme, name);
+}
+
+inline static void init_rox_theme(void)
+{
+	init_aux_theme(&rox_theme, "ROX");
+}
+
+inline static void init_gnome_theme(void)
+{
+	init_aux_theme(&gnome_theme, "gnome");
+}
+
+/* We don't want ROX to override configured theme so try all possibilities
+ * in icon_theme first */
+static GtkIconInfo *mime_type_lookup_icon_info(GtkIconTheme *theme,
+		MIME_type *type)
+{
+	char *type_name = g_strconcat("mime-", type->media_type, ":",
+				type->subtype, NULL);
+	GtkIconInfo *full = gtk_icon_theme_lookup_icon(theme, type_name,
+			HUGE_HEIGHT, 0);
+
+	g_free(type_name);
+	if (!full)
+	{
+		/* Ugly hack... try for a GNOME icon */
+		if (type == inode_directory)
+			type_name = g_strdup("gnome-fs-directory");
+		else
+			type_name = g_strconcat("gnome-mime-", type->media_type,
+					"-", type->subtype, NULL);
+		full = gtk_icon_theme_lookup_icon(theme, type_name, HUGE_HEIGHT, 0);
+		g_free(type_name);
+	}
+	if (!full)
+	{
+		/* Try for a media type */
+		type_name = g_strconcat("mime-", type->media_type, NULL);
+		full = gtk_icon_theme_lookup_icon(theme, type_name, HUGE_HEIGHT, 0);
+		g_free(type_name);
+	}
+	if (!full)
+	{
+		/* Ugly hack... try for a GNOME default media icon */
+		type_name = g_strconcat("gnome-mime-", type->media_type, NULL);
+
+		full = gtk_icon_theme_lookup_icon(theme, type_name, HUGE_HEIGHT, 0);
+		g_free(type_name);
+	}
+	return full;
+}
+
 /*			Actions for types 			*/
 
 /* Return the image for this type, loading it if needed.
@@ -371,6 +446,9 @@ MIME_type *mime_type_lookup(const char *type)
  * 3. Icon theme 'mime-base'
  * 4. Unknown type icon.
  *
+ * Special case: If an icon cannot be found for inode/mount-point, the icon for
+ * inode/directory will be returned (if possible).
+ *
  * Note: You must g_object_unref() the image afterwards.
  */
 MaskedPixmap *type_to_icon(MIME_type *type)
@@ -399,6 +477,7 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 		type->image = NULL;
 	}
 
+again:
 	type_name = g_strconcat(type->media_type, "_", type->subtype,
 				".png", NULL);
 	path = choices_find_xdg_path_load(type_name, "MIME-icons", SITE);
@@ -412,41 +491,22 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 	if (type->image)
 		goto out;
 
-	type_name = g_strconcat("mime-", type->media_type, ":",
-				type->subtype, NULL);
-	full = gtk_icon_theme_lookup_icon(icon_theme, type_name, HUGE_HEIGHT, 0);
-	g_free(type_name);
-	if (!full)
+	full = mime_type_lookup_icon_info(icon_theme, type);
+	if (!full && icon_theme != rox_theme)
 	{
-		/* Ugly hack... try for a GNOME icon */
-                if (type == inode_directory)
-			type_name = g_strdup("gnome-fs-directory");
-		else
-			type_name = g_strconcat("gnome-mime-", type->media_type,
-					"-", type->subtype, NULL);
-		full = gtk_icon_theme_lookup_icon(icon_theme,
-						type_name,
-						HUGE_HEIGHT, 0);
-		g_free(type_name);
+		init_rox_theme();
+		full = mime_type_lookup_icon_info(rox_theme, type);
 	}
-	if (!full)
+	if (!full && icon_theme != gnome_theme)
 	{
-		/* Try for a media type */
-		type_name = g_strconcat("mime-", type->media_type, NULL);
-		full = gtk_icon_theme_lookup_icon(icon_theme,
-						type_name,
-						HUGE_HEIGHT, 0);
-		g_free(type_name);
+		init_gnome_theme();
+		full = mime_type_lookup_icon_info(gnome_theme, type);
 	}
-	if (!full)
+	if (!full && type == inode_mountpoint)
 	{
-		/* Ugly hack... try for a GNOME default media icon */
-		type_name = g_strconcat("gnome-mime-", type->media_type, NULL);
-
-		full = gtk_icon_theme_lookup_icon(icon_theme,
-				type_name,
-				HUGE_HEIGHT, 0);
-		g_free(type_name);
+		/* Try to use the inode/directory icon for inode/mount-point */
+		type = inode_directory;
+		goto again;
 	}
 	if (full)
 	{
@@ -1191,56 +1251,64 @@ const char *mime_type_comment(MIME_type *type)
 	return type->comment;
 }
 
+static void unref_icon_theme(void)
+{
+	if (icon_theme && icon_theme != rox_theme && icon_theme != gnome_theme)
+		g_object_unref(icon_theme);
+}
+
 static void set_icon_theme(void)
 {
-	GtkIconInfo *info;
+	struct stat info;
 	char *icon_home;
+	const char *theme_dir;
 	const char *theme_name = o_icon_theme.value;
 
 	if (!theme_name || !*theme_name)
 		theme_name = "ROX";
 
-	while (1)
+	if (!strcmp(theme_name, "ROX"))
 	{
+		unref_icon_theme();
+		init_rox_theme();
+		icon_theme = rox_theme;
+	}
+	else if (!strcmp(theme_name, "gnome"))
+	{
+		unref_icon_theme();
+		init_gnome_theme();
+		icon_theme = gnome_theme;
+	}
+	else
+	{
+		if (icon_theme == rox_theme || icon_theme == gnome_theme)
+			icon_theme = gtk_icon_theme_new();
 		gtk_icon_theme_set_custom_theme(icon_theme, theme_name);
-		info = gtk_icon_theme_lookup_icon(icon_theme,
-				"mime-application:postscript",
-				ICON_HEIGHT, 0);
-		if (!info)
-		{
-			info = gtk_icon_theme_lookup_icon(icon_theme,
-					"gnome-mime-application-postscript",
-					ICON_HEIGHT, 0);
-		}
-		if (info)
-		{
-			gtk_icon_info_free(info);
-			return;
-		}
-
-		if (strcmp(theme_name, "ROX") == 0)
-			break;
-
-		delayed_error(_("Icon theme '%s' does not contain MIME icons. "
-				"Using ROX default theme instead."),
-				theme_name);
-		
-		theme_name = "ROX";
 	}
 
-	icon_home = g_build_filename(home_dir, ".icons", NULL);
-	if (!file_exists(icon_home))
-		mkdir(icon_home, 0755);
-	g_free(icon_home);	
+	/* Ensure the ROX theme exists. */
 
 	icon_home = g_build_filename(home_dir, ".icons", "ROX", NULL);
+	if (stat(icon_home, &info) == 0)
+		return;	/* Already exists */
+
+	/* First, create the .icons directory */
+	theme_dir = make_path(home_dir, ".icons");
+	if (!file_exists(theme_dir))
+		mkdir(theme_dir, 0755);
+
+	if (lstat(icon_home, &info) == 0)
+	{
+		/* Probably a broken symlink, then. Remove it. */
+		if (unlink(icon_home))
+			g_warning("Error removing broken symlink %s: %s", icon_home, g_strerror(errno));
+		else
+			g_warning("Removed broken symlink %s", icon_home);
+	}
+
 	if (symlink(make_path(app_dir, "ROX"), icon_home))
 	{
-		delayed_error(_("Failed to create symlink '%s':\n%s\n\n"
-		"(this may mean that the ROX theme already exists there, but "
-		"the 'mime-application:postscript' icon couldn't be loaded for "
-		"some reason, or %s is a link to an invalid directory; try "
-		"deleting it)"), icon_home, g_strerror(errno), icon_home);
+		delayed_error(_("Failed to create symlink '%s':\n%s"), icon_home, g_strerror(errno));
 		open_to_show(icon_home);
 	}
 	g_free(icon_home);
@@ -1377,3 +1445,46 @@ static GList *build_icon_theme(Option *option, xmlNode *node, guchar *label)
 
 	return g_list_append(NULL, hbox);
 }
+
+GtkIconInfo *theme_lookup_icon(const gchar *icon_name, gint size,
+		GtkIconLookupFlags flags)
+{
+	GtkIconInfo *result = gtk_icon_theme_lookup_icon(icon_theme,
+			icon_name, size, flags);
+
+	if (!result && icon_theme != rox_theme)
+	{
+		init_rox_theme();
+		result = gtk_icon_theme_lookup_icon(rox_theme,
+			icon_name, size, flags);
+	}
+	if (!result && icon_theme != gnome_theme)
+	{
+		init_gnome_theme();
+		result = gtk_icon_theme_lookup_icon(gnome_theme,
+			icon_name, size, flags);
+	}
+	return result;
+}
+
+GdkPixbuf *theme_load_icon(const gchar *icon_name, gint size,
+		GtkIconLookupFlags flags, GError **perror)
+{
+	GdkPixbuf *result = gtk_icon_theme_load_icon(icon_theme,
+			icon_name, size, flags, NULL);
+
+	if (!result && icon_theme != gnome_theme)
+	{
+		init_gnome_theme();
+		result = gtk_icon_theme_load_icon(gnome_theme,
+			icon_name, size, flags, NULL);
+	}
+	if (!result && icon_theme != rox_theme)
+	{
+		init_rox_theme();
+		result = gtk_icon_theme_load_icon(rox_theme,
+			icon_name, size, flags, perror);
+	}
+	return result;
+}
+
